#!/usr/bin/python
#
# Copyright (C) 2012 Intel Corporation
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Authors:
#              Tang, Shaofeng <shaofeng.tang@intel.com>

import ConfigParser
import os,os.path
import zipfile
import subprocess
import time
import shutil
from commodule.killall import killall

CONFIG_FILE = "/opt/tct/shell/CONFIG"

parser = ConfigParser.ConfigParser()
parser.read(CONFIG_FILE)

class Constants:

     TEST_SUITE_DIR  = parser.get('TCTSHELL', 'TEST_SUITE_DIR')

     WRT_LAUNCHR_CMD = parser.get('TCTSHELL', 'WRT_LAUNCHR_CMD')

     RELEASED_SUITE_LOCATION = TEST_SUITE_DIR + "%s"

     TCT_SHELL_HOME = parser.get('TCTSHELL', 'TCT_SHELL_HOME')

     TEMP_UNPASSED_XML_FOLDER = TCT_SHELL_HOME + "/tmp/"

     TEMP_RESULT_XML_FOLDER = TCT_SHELL_HOME + "/result/"

     if not TEMP_UNPASSED_XML_FOLDER.endswith('/'):
            TEMP_UNPASSED_XML_FOLDER = TEMP_UNPASSED_XML_FOLDER + "/"
     
     RERUN_OUTPUT = " -o " + TEMP_UNPASSED_XML_FOLDER + "result.xml"

     CAPABILITY_SUITE_NAME = "capability-tests"

     TCT_PLAN_FOLDER = parser.get('TCTSHELL', 'TCT_PLAN_FOLDER')

     STYLE_FOLDER = TCT_SHELL_HOME + "/style/*"

     CAPABILITY_PATH = TCT_SHELL_HOME + "/tmp/capability.xml"

     BUILD_INFO_PATH = TCT_SHELL_HOME + "/tmp/buildinfo.xml"

     DEVICE_CAPABILITY_PATH = parser.get('Device', 'DEVICE_CAPABILITY_PATH')

     DEVICE_BUILD_INFO_PATH = parser.get('Device', 'DEVICE_BUILD_INFO_PATH')

     DEVICE_DEFAULT_CONF = parser.get('Device', 'DEVICE_DEFAULT_CONF_PATH')
     
     DEVICE_DEFAULT_JSON_CONF = parser.get('Device', 'DEVICE_DEFAULT_ISON_CONF_PATH')

     DEVICE_HEALTH_CHECK_CMD = "/opt/tct/scripts/tct-config-device.sh --check"

     PRE_CONFIGURE_FILE_PATH = "/opt/tct/shell/tmp/pre_configure.conf"
     
     PRE_CONFIGURE_ISON_FILE_PATH = "/opt/tct/shell/tmp/preconfigure.json"

     DEVICE_CONFIGURE_FILE_PATH = parser.get('Device','DEVICE_PRECONFIGURE_FILE_PATH')

     TCT_TESTCONFIG_PATH = "/opt/usr/media/tct/opt/tct-testconfig/tests.xml"

     HEALTH_CHECK_FILE_PATH = "/opt/tct/scripts/healthcheck.ini"

     #PARAM Option
     XMLFILE_PREFF = "-f device:"
     EXECUTE_PREFF = " -e "
     ONLY_AUTO_CASES = "-A"
     ONLY_MANUAL_CASES = "-M"
     
     #SDB Command
     SDB_SHELL_TIMEOUT = 90
     SDB_PUSH = "sdb %s push"
     SDB_SHELL = "sdb %s shell"
     SDB_DEVICES = "sdb devices"
     SDB_PULL = "sdb %s pull"
     SDB_CAT_CONFIG = " cat %s"
     SDB_GET_DLOG_PRO = SDB_SHELL + " dlogctrl get platformlog"

     
     #DEVICE
     DEVICE_TMP_FOLDER = parser.get('Device', 'DEVICE_TMP_FOLDER')
     DEVICE_ZIP_PATH = "/opt/usr/media/tct"
     DEVICE_TESTS_FILE = parser.get('Device', 'DEVICE_TESTS_FILE') + " "

     #Testkit-Lite
     LITE_HOME = parser.get('TESTKIT_LITE', 'LITE_HOME')

     INSTALL_RPM       = "rpm -i --nodeps /tmp/%s"
     UNINSTALL_RPM     = "rpm -e --nodeps %s"
     REMOVE_RPM        = "rm -f /tmp/%s"
     INSTALL_ZIP = "inst.sh"
     UNINTALL_ZIP = "uninst.sh"

     #Plan Generator
     DEFAULT_MATCH_REGEX = "*.zip"
     DEFAULT_UNMATCH_REGEX = None
     DEFAULT_PLAN_NAME = "Full_test"
     DEFAULT_EXECUTE_TYPE = "All"
     DEFAULT_PLAN_FILE = "tct_plan"
     
     REGEX_FOR_SPLIT_PKG_NAME = "-\d\.\d\.\d"

     #Running Mode
     RUNNING_MODE_PLAN = "plan"
     RUNNING_MODE_RESULT = "result"
     RUNNING_MODE_SUITES = "suites"

     @staticmethod
     def unzip_package(filepath):
         path = os.path.expanduser("~")
         os.chdir(path)
         if os.path.exists(Constants.DEFAULT_PLAN_FILE):
             shutil.rmtree(Constants.DEFAULT_PLAN_FILE)
         os.makedirs(Constants.DEFAULT_PLAN_FILE)
         f = zipfile.ZipFile(filepath,'r')
         #print "new path: ", new_path
         for file in f.namelist():
             f.extract(file,path + "/"+Constants.DEFAULT_PLAN_FILE)

     @staticmethod
     def clean_unzipf_ile():
         os.chdir(os.path.expanduser("~"))
         shutil.rmtree(Constants.DEFAULT_PLAN_FILE)
         
     @staticmethod
     def add_right_enbrace(param):
         param = param.rstrip()
         param += "\""
         return param

     @staticmethod
     def indent(elem, level=0):
         i = "\n" + level*"  "
         if len(elem):
             if not elem.text or not elem.text.strip():
                 elem.text = i + "  "
             if not elem.tail or not elem.tail.strip():
                 elem.tail = i
             for elem in elem:
                 Constants.indent(elem, level+1)
             if not elem.tail or not elem.tail.strip():
                 elem.tail = i
         else:
             if level and (not elem.tail or not elem.tail.strip()):
                 elem.tail = i

     @staticmethod
     def copy_style_in_result_folder(result_folder,wrapper):
         os.system("cp -r " + Constants.STYLE_FOLDER + " '" + result_folder + "'")
         print "Going to open the result report with FireFox, if Firefox is not available on your machine, please copy the folder '%s' to a Firefox machine." % (result_folder + "summary.xml")
         print "If you want to rerun,please run cmd: tct-shell --rerun-fail '%s'" % (result_folder + "result.xml") + wrapper.is_manual_or_all_param()
         os.system("firefox '%s'summary.xml &" % result_folder)
    
     @staticmethod
     def execute_sdb_command(command):
         proc = subprocess.Popen(command,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
         time_out = 0
         result = []
         exit_code = None
         while time_out < Constants.SDB_SHELL_TIMEOUT:
             exit_code = proc.poll()
             if exit_code is not None:break
             time_out += 0.5
             time.sleep(0.5)
         if exit_code is None:
             print "sdb time out,kill subprocess..."
             killall(proc.pid)
         else:
             result = proc.stdout or proc.stderr
        
         return exit_code, result

