#!/usr/bin/python
#
# Copyright (C) 2012 Intel Corporation
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Authors:
#              Tang, Shaofeng <shaofeng.tang@intel.com>

from constants import Constants
from xml.etree import ElementTree
import xml.etree.ElementTree as etree
import os,os.path
import zipfile
import glob
import re

class SuitePackage:
    def __init__(self, file_name, command,auto_count,manual_count):
        self.file = file_name
        m = re.split(Constants.REGEX_FOR_SPLIT_PKG_NAME, file_name)
        self.name = m[0]
        self.auto_count = auto_count
        self.manual_count = manual_count
        self.total_count = auto_count + manual_count
        self.command = command + " %s" % self.name

    def to_xml(self):
        suite = ElementTree.Element('suite')
        suite.set('name', self.name)
        suite.set('launcher', Constants.WRT_LAUNCHR_CMD)
        auto = ElementTree.Element('auto_tcn')
        auto.text = str(self.auto_count)
        suite.append(auto)
        manual = ElementTree.Element('manual_tcn')
        manual.text = str(self.manual_count)
        suite.append(manual)
        total = ElementTree.Element('total_tcn')
        total.text = str(self.total_count)
        suite.append(total)
        rpm_name = ElementTree.Element('pkg_name')
        rpm_name.text = self.file
        suite.append(rpm_name)
        return suite

class PlanRunner:

    def __init__(self):
        self.suites = {}
        self.plan_name = Constants.DEFAULT_PLAN_NAME
        self.execute_type = Constants.DEFAULT_EXECUTE_TYPE

    def load_local_repo(self, path, match, command, unmatch,plan_name,execute_type):
        try:
            self.plan_name = plan_name
            self.execute_type = execute_type
            os.chdir(path)
            print "match: %s" % match
            print "unmatch: %s" % unmatch
            for files in glob.glob(match):
                print "File name: %s" % str(files)
                if (unmatch is not None) and (re.search(unmatch, str(files))):
                    print "File %s is skipped with unmatch regex %s" % (str(files), unmatch)
                else:
                    auto,manual = self.get_count(files,path +  str(files))
                    suite = SuitePackage(files, command,auto,manual)
                    self.suites[suite.name] = suite
                    Constants.clean_unzipf_ile();
                    
        except Exception, e:
            print "[ Error happen when reading the local repository, error: %s ]\n" % e

    def get_count(self,filesname,filepath):
         Constants.unzip_package(filepath)
         m = re.split(Constants.REGEX_FOR_SPLIT_PKG_NAME, filesname)
         name = m[0]
         test_file = os.path.expanduser("~") + "/"+Constants.DEFAULT_PLAN_FILE+"/opt/" + name + "/tests.xml"
         if not os.path.exists(test_file):
            print("Can't find the test file...")
            return 0,0
         return self.__read_test_xml(test_file)
         

    def __read_test_xml(self,test_file):
        autocnt = manualcnt = 0
        test_xml_temp = etree.parse(test_file)

        for test_xml_temp_suite in test_xml_temp.getiterator('suite'):
            suite_name = test_xml_temp_suite.get('name')
            autocnt = manualcnt = 0
            for tset in test_xml_temp_suite.getiterator('set'):
                set_autocnt, set_manualcnt = self.__get_set_casecnt(tset)
                autocnt += set_autocnt
                manualcnt += set_manualcnt
        return autocnt, manualcnt
     

    def __get_set_casecnt(self,tset):
         set_autocnt = set_manualcnt = 0
         for tcase in tset.getiterator('testcase'):
             exetype = tcase.get('execution_type')
             if exetype == "auto":
                 set_autocnt += 1
             elif exetype == "manual":
                 set_manualcnt += 1
         return set_autocnt, set_manualcnt

    def to_xml(self, xml_name):
        print "generating plan to %s" % xml_name
        root = ElementTree.Element('ns3:testplan')
        root.set('name', self.plan_name)
        root.set('xmlns','')
        root.set('xmlns:ns3', 'http://www.example.org/plan/')
        element = ElementTree.Element('execute_type')
        element.text = self.execute_type
        root.append(element)
        for suite_name in self.suites:
            suite = self.suites[suite_name]
            root.append(suite.to_xml())
        Constants.indent(root)
        tree = ElementTree.ElementTree()
        tree._setroot(root)
        tree.write(xml_name, encoding="utf-8")
