#!/bin/bash
#set -x

DIRP=`readlink -f $0`
PWD=`dirname $DIRP`
TOOLS_DIR=$PWD/../tools
RESRC_DIR=$PWD/../resource
PKGS_DIR=$PWD/../package
SERNO=""
EXE_MODE="--install"
WRT_INSTALLER="pkgcmd"


function show_usage(){
    echo "Usage: $0 command [parameters]"
    echo "commands:"
    echo "    $0 --install [deviceid=<serial_no>]                   - Install tct resources,'--install' can be omitted."
    echo "    $0 --purge [deviceid=<serial_no>]                     - Clean up tct resources."
    echo "    $0 --check [deviceid=<serial_no>] [proc=<proc_name>]  - Check tct resources."
    exit 0
}

function sdb_root_on(){
    echo "set sdb root on. Please wait..."
    sdb $SERNO root on
    if [ $? -ne 0 ]; then
        echo "Error: set root mode for Tizen device failed."
        echo "       Please confirm your device connected to machine and flashed with proper TIZEN image"
        echo "       And then re-run tct-config-device.sh script"
        exit 255
    fi
}

function check_resource_dir(){
    echo "check resource directory. Please wait..."
    if [ -d $RESRC_DIR ]; then
        echo "The directory resource exists"
    else
        echo "ERROR: The directory resource does not exists. Please copy test resources to $RESRC_DIR" 
        exit 255
    fi
    sdb $SERNO shell "mkdir -p /opt/usr/media/tct/"
    sdb $SERNO shell "mkdir -p /opt/usr/media/Documents/"
}

function install_testconfig(){
    echo "Install testconfig. Please wait..."
    RPMPKG=`ls $PKGS_DIR/ | grep tct-testconfig`
    sdb $SERNO push $PKGS_DIR/$RPMPKG /opt/usr/media/tct/$RPMPKG > /dev/null 2>&1
    sdb $SERNO shell "unzip -u /opt/usr/media/tct/$RPMPKG -d /opt/usr/media/tct/" > /dev/null 2>&1
    sdb $SERNO shell "/opt/usr/media/tct/opt/tct-testconfig/inst.sh"
    CHECKCONFIG=`sdb $SERNO shell "cat /opt/usr/media/tct/opt/tct-testconfig/tct-testconfig.ini 2>/dev/null | grep tct" `
    if [ -z "$CHECKCONFIG" ]; then
        echo "Failed to install testconfig, please check $PKGS_DIR."
        exit 255
    fi
}

function install_testconfig_ext(){
    echo "Install testconfig. Please wait..."
    RPMPKG=`ls $RESRC_DIR/ | grep tct-testconfig`
    sdb $SERNO push $RESRC_DIR/$RPMPKG /opt/usr/media/tct/$RPMPKG > /dev/null 2>&1
    sdb $SERNO shell "unzip -u /opt/usr/media/tct/$RPMPKG -d /opt/usr/media/tct/" > /dev/null 2>&1
    sdb $SERNO shell "/opt/usr/media/tct/opt/tct-testconfig/inst.sh"
    CHECKCONFIG=`sdb $SERNO shell "cat /opt/usr/media/tct/opt/tct-testconfig/tct-testconfig.ini 2>/dev/null | grep tct" `
    if [ -z "$CHECKCONFIG" ]; then
        echo "Failed to install testconfig, please check $RESRC_DIR."
        exit 255
    fi
}

function install_behavior(){
    echo "Install behavior tool. Please wait..."
    RPMPKG=`ls $PKGS_DIR/ | grep tct-behavior-tests` || `ls $PKGS_DIR/ | grep tct-behavior-tests`
    sdb $SERNO push $PKGS_DIR/$RPMPKG /opt/usr/media/tct/$RPMPKG > /dev/null 2>&1
    sdb $SERNO shell "unzip -u /opt/usr/media/tct/$RPMPKG -d /opt/usr/media/tct/" > /dev/null 2>&1
    sdb $SERNO shell "/opt/usr/media/tct/opt/tct-behavior-tests/inst.sh"
}

function uninstall_testconfig(){
    echo "Uninstall testconfig. Please wait..." 
    sdb $SERNO shell "/opt/usr/media/tct/opt/tct-testconfig/inst.sh -u"  > /dev/null 2>&1
}

function uninstall_behavior(){
    echo "Uninstall behavior tool. Please wait..."    
    sdb $SERNO shell "/opt/usr/media/tct/opt/tct-behavior-tests/inst.sh -u"  > /dev/null 2>&1
}

function check_testconfig(){
    CHECKCONFIG=`sdb $SERNO shell "cat /opt/usr/media/tct/opt/tct-testconfig/tct-testconfig.ini 2>/dev/null | grep tct" `
    if [ -z "$CHECKCONFIG" ]; then
        echo "no testconfig in device. Now Install it"
        install_testconfig_ext
    else
        echo "testconfig has already been installed on device."
    fi
}

function install_widget(){
    echo "Install widget $1. Please wait..."
    WIDGETPKG=`ls $RESRC_DIR/ | grep $1`
    sdb $SERNO push $RESRC_DIR/$WIDGETPKG /opt/usr/media/tct/$WIDGETPKG > /dev/null 2>&1
    sdb $SERNO shell "$WRT_INSTALLER  -i -t wgt -q -p /opt/usr/media/tct/$WIDGETPKG "
}

function uninstall_widget(){
    echo "Uninstall widget $1. Please wait..."
    WGTNAME=`sdb $SERNO shell wrt-launcher -l | grep $1 | awk '{ print $(NF-1) }' | sed 's/[[:space:]]*$//'`
    if [ -z $WGTNAME ]; then
        echo "Widget $1 is not installed. No need to uninstall"
    else
        echo "Try to uninstall the widget $1..."
        UNINSTL=`sdb $SERNO shell "$WRT_INSTALLER -u -q -t wgt -n $WGTNAME" | grep [ok]`
        if [ -z "$UNINSTL" ]; then
            echo "Uninstall widget $1 failed. Please uninstall it manually via following command:"
            echo "   sdb $SERNO shell"
            echo "   $WRT_INSTALLER -u -q -t wgt -n $WGTNAME"
        else
            echo "Widget $1 uninstalled successfully"
        fi
    fi
}

function kill_process(){
    echo "Kill process $1. Please wait..."
    PRID=`sdb $SERNO shell "ps aux | grep $1 | grep -v grep" | awk '{print $2}'  `
    if [ -z "$PRID" ]; then 
        echo "No process of $1 activated"
    else
        sdb $SERNO shell "kill -9 $PRID"
        echo "$1 process has been killed"
    fi
}

function remove_tinyweb(){
    echo "Clean tinyweb. Please wait..."
    sdb $SERNO shell "rm -f /opt/home/developer/server.pem" > /dev/null 2>&1
    sdb $SERNO shell "rm -f /opt/home/developer/tinyweb" > /dev/null 2>&1
    sdb $SERNO shell "rm -f /opt/home/developer/cgi-getcookie" > /dev/null 2>&1
    sdb $SERNO shell "rm -f /opt/home/developer/cgi-getfield" > /dev/null 2>&1
    sdb $SERNO shell "rm -f /opt/home/developer/libmongoose.so" > /dev/null 2>&1
    sdb $SERNO shell "rm -f /opt/home/developer/echo.so" > /dev/null 2>&1
}

function remove_stub(){
    echo "Clean testkit-stub. Please wait..."
    sdb $SERNO shell "rm -f /opt/home/developer/testkit-stub" > /dev/null 2>&1
}

function remove_results(){
    echo "Clean result files. Please wait..."
    sdb $SERNO shell "rm -rf /opt/usr/media/Documents/tct/" > /dev/null 2>&1
    sdb $SERNO shell "rm -rf /opt/usr/media/Documents/tct-behavior*" > /dev/null 2>&1
}

function install_stub(){
    echo "Install testkit-stub. Please wait..."
    TESTSTUB=`ls $RESRC_DIR/ | grep testkit-stub`
    sdb $SERNO push $RESRC_DIR/$TESTSTUB /opt/home/developer/$TESTSTUB > /dev/null 2>&1
    sdb $SERNO shell "chmod +x /opt/home/developer/testkit-stub" > /dev/null 2>&1    
}

function install_tinyweb(){
    echo "Install tinyweb. Please wait..."
    TINYWEB=`ls $RESRC_DIR/ | grep tinyweb`
    sdb $SERNO push $RESRC_DIR/$TINYWEB/tinyweb /opt/home/developer/ > /dev/null 2>&1 
    sdb $SERNO shell "chmod a+x /opt/home/developer/tinyweb" > /dev/null 2>&1
    sdb $SERNO push $RESRC_DIR/$TINYWEB/cgi-getcookie /opt/home/developer/ > /dev/null 2>&1
    sdb $SERNO shell "chmod a+x /opt/home/developer/cgi-getcookie" > /dev/null 2>&1
    sdb $SERNO push $RESRC_DIR/$TINYWEB/cgi-getfield /opt/home/developer/ > /dev/null 2>&1
    sdb $SERNO shell "chmod a+x /opt/home/developer/cgi-getfield" > /dev/null 2>&1
    sdb $SERNO push $RESRC_DIR/$TINYWEB/libmongoose.so /opt/home/developer/ > /dev/null 2>&1
    sdb $SERNO shell "chmod 666 /opt/home/developer/libmongoose.so" > /dev/null 2>&1
    sdb $SERNO push $RESRC_DIR/$TINYWEB/echo.so /opt/home/developer/  > /dev/null 2>&1
    sdb $SERNO shell "chmod 666 /opt/home/developer/echo.so" > /dev/null 2>&1
    sdb $SERNO push $RESRC_DIR/$TINYWEB/server.pem /opt/home/developer/  > /dev/null 2>&1
    sdb $SERNO shell "chmod 666 /opt/home/developer/server.pem" > /dev/null 2>&1
    sdb $SERNO shell "ln -s /usr/lib/libssl.so.1.0.0 /opt/home/developer/libssl.so" > /dev/null 2>&1
    sdb $SERNO shell "ln -s /usr/lib/libcrypto.so.1.0.0 /opt/home/developer/libcrypto.so" > /dev/null 2>&1
}

function launch_tinyweb(){
    DPATH=`sdb $SERNO shell "printenv PATH"`
    timeout 5 sdb $SERNO shell "env LD_LIBRARY_PATH=/opt/home/developer PATH=$DPATH:/opt/home/developer tinyweb -ssl_certificate /opt/home/developer/server.pem -document_root /opt/usr/media/tct/ -listening_ports 80,8080,8081,8082,8083,8443s; sleep 3s" > /dev/null 2>&1
    TINYWEBSTATUS=`sdb $SERNO shell "ps aux | grep tinyweb | grep -v grep"`
    if [ -z "$TINYWEBSTATUS" ]; then 
        echo "Active process tinyweb failed."
        exit 255
    else
        echo "Active process tinyweb successfully."
    fi
}

function launch_widget(){
    echo "Check widget $1. Please wait..."
    WGTID=`sdb $SERNO shell wrt-launcher -l | grep $1 | awk '{ print $NF }' | sed 's/[[:space:]]*$//'`
    if [ -z "$WGTID" ]; then
        echo "Widget $1 cannot be found. Please check whether the widget is installed successfully"
        exit 255
    else
        echo "Try to launch the $1 widget..."
        LAUNCHRESULT=`sdb $SERNO shell "wrt-launcher -s $WGTID" | grep launched`
        if [ -z "$LAUNCHRESULT" ]; then 
            echo "Launch $1 got failed"
            exit 255
        else
            echo "Launch $1 successfully"
        fi
    fi    
}

function check_getCap(){
    echo "Check getCap widget. Please wait..."
    WIDGETID=`sdb $SERNO shell wrt-launcher -l | grep getCap | awk '{ print $NF }' | sed 's/[[:space:]]*$//'`
    if [ -z "$WIDGETID" ]; then
        install_widget "getCap"
    else
        echo "widget getCap has already been installed on device."
    fi
    launch_widget "getCap" && sleep 5s
    CAPRESULT=`sdb $SERNO shell "cat /opt/usr/media/Documents/tct/buildinfo.xml 2>/dev/null | grep buildinfos" `
    if [ -z "$CAPRESULT" ]; then
        echo "getCap get buildinfo of device failed!"
        exit 255
    fi
}

function check_tinyweb(){
    echo "Check tinyweb. Please wait..."
    CHECKTINYWEB=`sdb $SERNO shell "ls /opt/home/developer/ | grep tinyweb" `
    if [ -z "$CHECKTINYWEB" ]; then
        echo "no tinyweb in device. Now install tinyweb"
        install_tinyweb
    fi
    launch_tinyweb
}

function check_stub(){
    echo "Check testkit-stub. Please wait..."
    CHECKSTUBA=`sdb $SERNO shell "ls /opt/home/developer | grep testkit-stub" `
    if [ -z "$CHECKSTUBA" ]; then
        install_stub
    else
        echo "testkit-stub has been installed on device."
    fi
}

if [ $# -ge 1 ] ; then
    for arg in "$@"
    do
        if [ "$arg"x = "-h"x -o "$arg"x = "--help"x ]; then
            show_usage
        fi
        checkstring=`echo $arg | grep deviceid`
        procstring=`echo $arg | grep proc`
        if [ -z "$checkstring" ]; then
            if [ "$arg"x = "-p"x -o "$arg"x = "--purge"x ]; then
                EXE_MODE="--purge"
            elif [ "$arg"x = "--check"x ]; then 
                EXE_MODE="--check"
            fi
        else
            export $arg
            SERNO="-s $deviceid"
        fi
        if [ -z "$procstring" ]; then
            if [ "$arg"x != "--check"x -a "$EXE_MODE" = "--check" ]; then
                if [ -z "$checkstring" ]; then
                    proc=""
                fi
            fi
        else
            export $arg
        fi
    done
fi


# init device
sdb_root_on

if [ $EXE_MODE = "--purge" ]; then
    echo "----------------------------------------------"
    echo "[ Uninstall test resource on device. Please wait...]"
    uninstall_testconfig
    uninstall_behavior
    uninstall_widget "getCap"
    kill_process "tinyweb" && remove_tinyweb
    kill_process "testkit-stub" && remove_stub
    remove_results
    echo "Clean the tct packages in device successfully."
    echo "----------------------------------------------"
    exit 0
elif [ $EXE_MODE = "--check" ]; then
    check_resource_dir
    echo "------------------------------------------------------------------------"
    echo "[ Check test resource on device. Please wait...]"
    case "$proc" in
    "tinyweb") check_tinyweb ;;
    "testkit-stub") check_stub ;;
    "testconfig") check_testconfig ;;
    "getCap") check_getCap ;;
    *)  check_tinyweb && check_stub && check_testconfig && check_getCap ;;
    esac
    echo "Check finished."
    exit 0
elif [ $EXE_MODE = "--install" ]; then
    check_resource_dir
    echo "------------------------------------------------------------------------"
    echo "[ Clean old test resource on device. Please wait...]"
    uninstall_testconfig
    uninstall_behavior
    uninstall_widget "getCap"
    kill_process "tinyweb" && remove_tinyweb
    kill_process "testkit-stub" && remove_stub
    remove_results
    echo "Clean the tct packages in device successfully."

    echo "------------------------------------------------------------------------"
    echo "[ Install test resource on device. Please wait...]"
    install_testconfig
    install_behavior
    install_stub
    install_tinyweb && launch_tinyweb
    install_widget "getCap" && launch_widget "getCap" && sleep 3s

    echo "All of installation process on device completed."
    exit 0
fi
